@@ -3,7 +3,7 @@
 #
 # This file is part of Pysilhouette.
 #
-# Copyright (c) 2009 HDE, Inc.
+# Copyright (c) 2009-2010 HDE, Inc.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -28,9 +28,13 @@
 @author: Kei Funagayama <kei@karesansui-project.info>
 """
 
-__version__ = '0.6'
-__release__ = '3'
+__version__ = '0.7'
+__release__ = '0'
 __app__ = 'pysilhouette'
 
+# DEFINE
+PROCERROR = 1
+PROCSUCCESS = 0
+
 class SilhouetteException(StandardError):
     pass
