@@ -3,7 +3,7 @@
 #
 # This file is part of Pysilhouette.
 #
-# Copyright (c) 2009 HDE, Inc.
+# Copyright (c) 2009-2010 HDE, Inc.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,7 @@
 
 import subprocess
 import os
+import sys
 import traceback
 import logging
 
@@ -50,14 +51,7 @@ class SilhouetteWorkerException(pysilhouette.SilhouetteException):
 class Worker:
     """Worker Base class
     """
-    
-    def __init__(self, cf, db, jobgroup_id):
-        self._cf = cf
-        self._db = db
-        self._jobgroup_id = jobgroup_id
-        self.logger = logging.getLogger('pysilhouette.performer.worker')
-    
-    def run(self):
+    def process(self):
         try:
             session = self._db.get_session()
             self.logger.debug('Session was obtained from the database. - session=%s' % session)
@@ -66,7 +60,6 @@ class Worker:
                                                self._cf['env.uniqkey'])
             
             if self._m_jg is None: return False
-            
             jobgroup_update(session, self._m_jg, JOBGROUP_STATUS['RUN']) # JobGroup UPDATE
             _m_jobs = job_findbyjobgroup_id(session, self._jobgroup_id, False) # order asc
 
@@ -76,12 +69,12 @@ class Worker:
             try:
                 ret = self._action(session, _m_jobs)
             except Exception, e:
-                self.logger.info('Failed to perform the job action. Exceptions are not expected. - jobgroup_id=%d : %s'
-                             % (self._jobgroup_id, str(e.args)))
-                
-                jobgroup_update(session, self._m_jg, JOBGROUP_STATUS['APPERR'])
                 t_logger = logging.getLogger('pysilhouette_traceback')
                 t_logger.info(traceback.format_exc())
+                self.logger.info('%s, Failed to perform the job action. Exceptions are not expected. - jobgroup_id=%d : %s, JobGroup status=%s'
+                             % (self.getName(), self._jobgroup_id, str(e.args), JOBGROUP_STATUS['APPERR']))
+
+                jobgroup_update(session, self._m_jg, JOBGROUP_STATUS['APPERR'])
                 err = True
 
             try:
@@ -192,6 +185,11 @@ class Worker:
 class SimpleWorker(Worker):
     """Sequential Worker Class
     """
+    def __init__(self, cf, db, jobgroup_id):
+        self._cf = cf
+        self._db = db
+        self._jobgroup_id = jobgroup_id
+        self.logger = logging.getLogger('pysilhouette.worker.simpleworker')
 
     def _action(self, session, m_jobs):
         ret = True
@@ -207,23 +205,33 @@ class SimpleWorker(Worker):
                 lcmd = split_shell_command(cmd)
                 if self.chk_whitelist(lcmd[0]):
                     try:
-                        (proc, proc_info) = popen(lcmd,
-                                                  self._cf['job.popen.timeout'],
-                                                  self._cf['job.popen.waittime'],
-                                                  self._cf['job.popen.env.lang'],
-                                                  m_job.STD_OUTPUT_LIMIT,
-                                                  m_job.id,
+                        (proc, proc_info) = popen(cmd=lcmd,
+                                                  timeout=self._cf['job.popen.timeout'],
+                                                  waittime=self._cf['job.popen.waittime'],
+                                                  lang=self._cf['job.popen.env.lang'],
+                                                  limit=self._cf['job.popen.output.limit'],
+                                                  job_id=m_job.id,
                                                   )
 
                         self.logger.debug('Of commands executed stdout=%s' % proc_info['stdout'])
                         self.logger.debug('Of commands executed stderr=%s' % proc_info['stderr'])
 
+                        if self._cf['job.popen.output.limit'] < len(proc_info['stdout']):
+                            self.logger.info("There was a limit beyond stdout output. Information-processing is truncated beyond the limit. - limit=%d, stdout=%d" \
+                                             % (self._cf['job.popen.output.limit'], len(proc_info['stdout'])))
+
+                        if self._cf['job.popen.output.limit'] < len(proc_info['stderr']):
+                            self.logger.info("There was a limit beyond stderr output. Information-processing is truncated beyond the limit. - limit=%d, stderr=%d" \
+                                             % (self._cf['job.popen.output.limit'], len(proc_info['stderr'])))
+                            
+
                     except OSError, oe:
                         self.logger.info('action command system failed!! job_id=%d : cmd=%s'
                                           % (m_job.id, cmd))
                         raise oe
-    
+
                     job_result_action(session, m_job, proc_info) # Job result UPDATE
+
                     if proc_info['r_code'] == 0: # Normal end
                         self.logger.info('action command was successful!! job_id=%d : cmd=%s'
                                           % (m_job.id, cmd))
@@ -266,11 +274,11 @@ class SimpleWorker(Worker):
                     
                     if self.chk_whitelist(lcmd[0]):
                         try:
-                            (proc, proc_info) = popen(lcmd,
-                                                      self._cf['job.popen.timeout'],
-                                                      self._cf['job.popen.waittime'],
-                                                      self._cf['job.popen.env.lang'],
-                                                      m_job.STD_OUTPUT_LIMIT,
+                            (proc, proc_info) = popen(cmd=lcmd,
+                                                      timeout=self._cf['job.popen.timeout'],
+                                                      waittime=self._cf['job.popen.waittime'],
+                                                      lang=self._cf['job.popen.env.lang'],
+                                                      limit=self._cf['job.popen.output.limit'],
                                                       )
 
                             self.logger.debug('Of commands executed stdout=%s' % proc_info['stdout'])
@@ -304,25 +312,162 @@ class SimpleWorker(Worker):
                 self.logger.debug('Does not rollback the process. - job_id=%d : status=%s'
                                   % (m_job.id, m_job.status))
 
+# --
+import threading
+
+class ThreadQueue(threading.Thread):
+    def __init__(self, request_queue, response_list, *args, **kwargs):
+        threading.Thread.__init__(self, *args, **kwargs)
+        self.setDaemon(1)
+        self.request_queue = request_queue
+        self.response_list = response_list
+        self.start()
+        self.logger = logging.getLogger('pysilhouette.worker.threadqueue')
+
+    def now_alive(self):
+        ret = 0
+        for th in self.response_list:
+            if th[0].isAlive() is True:
+                ret =+ 1
+        return ret
+
+    def response_clean(self):
+        _size = 0
+        _len = len(self.response_list)
+        while _size < _len:
+            if self.response_list[_size][0].isAlive() is False:
+                self.response_list.pop(_size) # remove
+                _len =- 1
+            else:
+                _size =+ 1
+        return len(self.response_list)
+
+    def put(self, callable, *args, **kwargs):
+        self.logger.debug('callable=%s' % str(callable))
+        self.request_queue.put((callable, args, kwargs))
+
+    def run(self):
+        while True:
+            callable, args, kwargs = self.request_queue.get()
+            self.logger.debug('callable=%s, args=%s, kwargs=%s' \
+                               % (str(callable), str(args), str(kwargs)))
+            callable.start()
+            self.response_list.append((callable, args, kwargs))
+
+class ThreadWorker(threading.Thread, SimpleWorker):
+    def __init__(self, cf, db, jobgroup_id):
+        threading.Thread.__init__(self)
+        self._cf = cf
+        self._jobgroup_id = jobgroup_id
+        self._db = db
+
+    def run(self):
+        self.logger = logging.getLogger('pysilhouette.worker.threadworker')
+        try:
+            self.process()
+        except Exception, e:
+            self.logger.error('%s - JobGroup execute failed. - jobgroup_id=%d : %s, JobGroup status=%s' \
+                              % (self.getName(), self._jobgroup_id, str(e.args), JOBGROUP_STATUS['APPERR']))
+            t_logger = logging.getLogger('pysilhouette_traceback')
+            t_logger.error(traceback.format_exc())
+            try:
+                session = self._db.get_session()
+                jobgroup_update(session,
+                                jobgroup_findbyid(session, self._jobgroup_id, self._cf['env.uniqkey']),
+                                JOBGROUP_STATUS['APPERR'])
+                session.close()
+            except:
+                self.logger.error('JobGroup failed to update. - jobgroup_id=%d : %s, update status=%s' \
+                              % (self._jobgroup_id, str(e.args), JOBGROUP_STATUS['APPERR']))
+                t_logger.error(traceback.format_exc())
+
+def dummy_set_job(cf, number, action, rollback, finish, type, db=None):
+    try:
+        if db is None:
+            db = Database(cf['database.url'],encoding="utf-8",convert_unicode=True,echo=True,echo_pool=True)
+            reload_mappers(db.get_metadata())
+
+        session = db.get_session()
+    except Exception, e:
+        print >>sys.stderr, 'Initializing a database error'
+        raise
+
+    try:
+        jgs = []
+        for i in range(number):
+            jg_name = u'%s-%d' % ('worker#dummy_set_job#jobgroup', i)
+            jg_ukey = unicode(cf['env.uniqkey'], "utf-8")
+            jg = JobGroup(jg_name, jg_ukey)
+            if not finish is None:
+                jg.finish_command = unicode(finish, "utf-8")
+            if type == 'serial':
+                jg.type = JOBGROUP_TYPE['SERIAL']
+            elif type == 'parallel':
+                jg.type = JOBGROUP_TYPE['PARALLEL']
+            else:
+                jg.type = JOBGROUP_TYPE['SERIAL']
+
+            j_name = u'%s-%d' % ('worker#dummy_set_job#job', i)
+            j_order = i
+            j = Job(j_name, j_order, unicode(action, "utf-8"))
+            if not rollback is None:
+                j.rollback_command = unicode(rollback, "utf-8")
+
+            jg.jobs.append(j)
+            jgs.append(jg)
+            
+        session.add_all(jgs)
+        session.commit()
+        session.close()
+        print >>sys.stdout, 'Insert JobGroup and Job. num=%d [OK]' % number
+    except Exception, e:
+        print >>sys.stderr, 'Failed to add JobGroup and Job.'
+        raise
+            
+
+
 if __name__ == '__main__':
-    """Testing
-    """
-    import pysilhouette.tests.testworker
-    # dev start
+    import Queue
+    request_queue = Queue.Queue()
+    #response_queue = Queue.Queue()
+    response_list = []
+    tq = ThreadQueue(request_queue, response_list)
     _env = os.environ
     _env['PYSILHOUETTE_CONF'] = '/etc/opt/pysilhouette/silhouette.conf'
-    # dev end
-
-    # init
     from pysilhouette.prep import readconf 
-    cf = readconf(os.environ['PYSILHOUETTE_CONF'])
+    #cf = readconf(os.environ['PYSILHOUETTE_CONF'])
     pysilhouette.cf = pysilhouette.prep.readconf(os.environ['PYSILHOUETTE_CONF'])
+    import pysilhouette
+    import pysilhouette.log
+    import sys
     pysilhouette.log.reload_conf(pysilhouette.cf["env.sys.log.conf.path"])
+    number = 50
+    dummy_set_job(pysilhouette.cf,number,'echo "aaaaaa"','echo "bbbbb"','echo "cccc"','parallel')
+    try:
+        db = Database(pysilhouette.cf['database.url'],encoding="utf-8",convert_unicode=True,echo=True,echo_pool=True)
+        reload_mappers(db.get_metadata())
+        session = db.get_session()
+    except Exception, e:
+        print >>sys.stderr, 'Initializing a database error'
+        raise
 
-    (db, session, _m_jgs) = pysilhouette.tests.testworker.test_setup(cf)
-    # run
-    for _m_jg in _m_jgs:
-        _w = SimpleWorker(db, _m_jg.id)
-        _w.run()
-        pysilhouette.tests.testworker.worker_debug(db, _m_jg.id)
-    session.close()
+    from pysilhouette.db.access import jobgroup_findbytype_status
+    m_jgs = jobgroup_findbytype_status(session, JOBGROUP_TYPE['PARALLEL'])
+    for m_jg in m_jgs:
+        try:
+            tq.put(ThreadWorker(pysilhouette.cf, db, m_jg.id))
+        except Exception, e:
+            import traceback
+            print traceback.format_exc()
+
+    import time
+    time.sleep(2)
+
+    while True:
+        size = tq.response_clean()
+        print 'ret=' + str(size)
+        if size <= 0:
+            break
+        time.sleep(2)
+
+    print 'end (request_queue size=%d, response_list=%s)' % (tq.request_queue.qsize(), tq.response_list)
